@@ -12,6 +12,7 @@ Several considerations are made;
      method. Arguments are packaged in _Arguments.
 """
 
+import SBstoat
 from SBstoat.namedTimeseries import NamedTimeseries, TIME, mkNamedTimeseries
 from SBstoat.timeseriesStatistic import TimeseriesStatistic
 from SBstoat._bootstrapResult import BootstrapResult
@@ -19,7 +20,7 @@ from SBstoat.observationSynthesizer import  \
       ObservationSynthesizerRandomizedResiduals
 from SBstoat import _modelFitterCore as mfc
 from SBstoat import _helpers
-from SBstoat._logger import Logger
+from SBstoat.logs import Logger
 
 import inspect
 import lmfit
@@ -50,14 +51,16 @@ class _Arguments():
                  reportInterval:int=-1,
                  synthesizerClass=  \
                  ObservationSynthesizerRandomizedResiduals,
+                 _loggerPrefix="",
                  **kwargs: dict):
         # Same the antimony model, not roadrunner bcause of Pickle
-        self.fitter = fitter.copy()
+        self.fitter = fitter.copy(isKeepLogger=True)
         self.numIteration  = numIteration
         self.reportInterval  = reportInterval
         self.numProcess = numProcess
         self.processIdx = processIdx
         self.synthesizerClass = synthesizerClass
+        self._loggerPrefix = _loggerPrefix
         self.kwargs = kwargs
        
 
@@ -74,12 +77,16 @@ def _runBootstrap(arguments:_Arguments, queue=None)->BootstrapResult:
     Notes
     -----
     1. Only the first process generates progress reports.
-        
+    2. Uses METHOD_LEASTSQ for fitModel iterations.
     """
+    fitter = arguments.fitter
+    logger = fitter.logger
+    mainBlock = Logger.join(arguments._loggerPrefix, "_runBootstrap")
+    mainGuid = logger.startBlock(mainBlock)
     # Unapack arguments
     isSuccess = False
-    fitter = arguments.fitter
     lastErr = ""
+    # Do an initial fit
     for _ in range(MAX_TRIES):
         try:
             fitter.fitModel()  # Initialize model
@@ -88,14 +95,15 @@ def _runBootstrap(arguments:_Arguments, queue=None)->BootstrapResult:
         except Exception as err:
             lastErr = err
     # Set up logging for this process
-    fd = fitter._logger.getFileDescriptor()
+    fd = logger.getFileDescriptor()
     processIdx = arguments.processIdx
     if fd is not None:
-        sys.stderr = fitter._logger.getFileDescriptor()
-        sys.stdout = fitter._logger.getFileDescriptor()
+        sys.stderr = logger.getFileDescriptor()
+        sys.stdout = logger.getFileDescriptor()
+    iterationGuid = None
     if not isSuccess:
         msg = "Process %d/modelFitterBootstrip/_runBootstrap" % processIdx
-        fitter._logger.error(msg,  lastErr)
+        logger.error(msg,  lastErr)
         fittedStatistic = TimeseriesStatistic(fitter.observedTS, percentiles=[])
         bootstrapResult = BootstrapResult(fitter, 0, {},
               fittedStatistic)
@@ -118,21 +126,29 @@ def _runBootstrap(arguments:_Arguments, queue=None)->BootstrapResult:
         baseChisq = fitter.minimizerResult.redchi
         # Do the bootstrap iterations
         bootstrapError = 0
+        iterationBlock = Logger.join(mainBlock, "Iteration")
         for iteration in range(numIteration*ITERATION_MULTIPLIER):
+            if iterationGuid is not None:
+                logger.endBlock(iterationGuid)
+            iterationGuid = logger.startBlock(iterationBlock)
             newObservedTS = synthesizer.calculate()
-            std = np.std(newObservedTS.flatten())
+            fittingSetupBlock = Logger.join(iterationBlock, "fittingSetup")
+            fittingSetupGuid = logger.startBlock(fittingSetupBlock)
             newFitter = ModelFitterBootstrap(fitter.roadrunnerModel,
                   newObservedTS,  
                   fitter.parametersToFit,
                   selectedColumns=fitter.selectedColumns,
-                  method=fitter._method,
+                  # Use bootstrap methods for fitting
+                  fitterMethods=fitter._bootstrapMethods,
                   parameterLowerBound=fitter.lowerBound,
                   parameterUpperBound=fitter.upperBound,
                   fittedDataTransformDct=fitter.fittedDataTransformDct,
-                  logger=fitter._logger,
+                  logger=logger,
+                  _loggerPrefix=iterationBlock,
                   isPlot=fitter._isPlot)
             fittedStatistic = TimeseriesStatistic(newFitter.observedTS,
                   percentiles=[])
+            logger.endBlock(fittingSetupGuid)
             try:
                 if (iteration > 0) and (iteration != lastReport)  \
                         and (processIdx == 0):
@@ -142,24 +158,33 @@ def _runBootstrap(arguments:_Arguments, queue=None)->BootstrapResult:
                         msg = "Bootstrap completed %d total iterations "
                         msg += "with %d successes."
                         msg = msg % (totalIteration, totalSuccessIteration)
-                        fitter._logger.status(msg)
+                        fitter.logger.status(msg)
                         lastReport = numSuccessIteration
                 if numSuccessIteration >= numIteration:
                     # Performed the iterations
                     break
+                tryBlock = Logger.join(iterationBlock, "try")
+                tryGuid = logger.startBlock(tryBlock)
                 try:
+                    tryFitterBlock = Logger.join(tryBlock, "Fitter")
+                    tryFitterGuid = logger.startBlock(tryFitterBlock)
                     newFitter.fitModel(params=fitter.params)
+                    logger.endBlock(tryFitterGuid)
                 except Exception as err:
                     # Problem with the fit. Don't numSuccessIteration it.
                     msg = "Process %d/modelFitterBootstrap" % processIdx
                     msg += " Fit failed on iteration %d."  % iteration
-                    fitter._logger.error(msg, err)
+                    fitter.logger.error(msg, err)
+                    logger.endBlock(tryGuid)
                     continue
                 if newFitter.minimizerResult.redchi > MAX_CHISQ_MULT*baseChisq:
                     if IS_REPORT:
                         msg = "Process %d: Fit has high chisq: %2.2f on iteration %d."
-                        fitter._logger.exception(msg % (processIdx,
+                        fitter.logger.exception(msg % (processIdx,
                               newFitter.minimizerResult.redchi, iteration))
+                    logger.endBlock(tryGuid)
+                    continue
+                if newFitter.params is None:
                     continue
                 numSuccessIteration += 1
                 dct = newFitter.params.valuesdict()
@@ -167,14 +192,18 @@ def _runBootstrap(arguments:_Arguments, queue=None)->BootstrapResult:
                 cols = newFitter.fittedTS.colnames
                 fittedStatistic.accumulate(newFitter.fittedTS)
                 newFitter.observedTS = synthesizer.calculate()
+                logger.endBlock(tryGuid)
             except Exception as err:
                 msg = "Process %d/modelFitterBootstrap" % processIdx
                 msg += " Error on iteration %d."  % iteration
-                fitter._logger.error(msg, err)
+                fitter.logger.error(msg, err)
                 bootstrapError += 1
-        fitter._logger.status("Process %d: completed bootstrap." % (processIdx + 1))
+        fitter.logger.status("Process %d: completed bootstrap." % (processIdx + 1))
         bootstrapResult = BootstrapResult(fitter, numSuccessIteration, parameterDct,
               fittedStatistic, bootstrapError=bootstrapError)
+    if iterationGuid is not None:
+        logger.endBlock(iterationGuid)
+    logger.endBlock(mainGuid)
     if fd is not None:
         if not fd.closed:
             fd.close()
@@ -242,10 +271,11 @@ class ModelFitterBootstrap(mfc.ModelFitterCore):
               numIteration=numProcessIteration,
               reportInterval=reportInterval,
               synthesizerClass=synthesizerClass,
+              _loggerPrefix="bootstrap",
               **kwargs) for i in range(numProcess)]
         msg = "Running bootstrap for %d successful iterations " % numIteration
         msg += "with %d processes." % numProcess
-        self._logger.activity(msg)
+        self.logger.activity(msg)
         # Run separate processes for each bootstrap
         processes = []
         queue = multiprocessing.Queue()
@@ -267,7 +297,7 @@ class ModelFitterBootstrap(mfc.ModelFitterCore):
                     process.terminate()
             except Exception as err:
                 msg = "modelFitterBootstrap/Error in process management"
-                self._logger.error(msg, err)
+                self.logger.error(msg, err)
             finally:
                 pass
         else:
@@ -279,12 +309,15 @@ class ModelFitterBootstrap(mfc.ModelFitterCore):
             msg = "modelFitterBootstrap/timeout in solving model."
             msg = "\nConsider increasing per timeout."
             msg = "\nCurent value: %f" % MAX_ITERATION_TIME
-            self._logger.result(msg)
+            self.logger.result(msg)
         else:
             self.bootstrapResult = BootstrapResult.merge(results)
+            # Update the logger in place
+            _ = _helpers.copyObject(self.bootstrapResult.fitter.logger,
+                  self.logger)
             if self.bootstrapResult.fittedStatistic is not None:
                 self.bootstrapResult.fittedStatistic.calculate()
-            self._logger.result("%d bootstrap estimates of parameters."
+            self.logger.result("%d bootstrap estimates of parameters."
                   % self.bootstrapResult.numSimulation)
             if serializePath is not None:
                 self.serialize(serializePath)
