@@ -8,7 +8,7 @@ Core logic of model fitter. Does not include plots.
 """
 
 from SBstoat.namedTimeseries import NamedTimeseries, TIME, mkNamedTimeseries
-from SBstoat._logger import Logger
+from SBstoat.logs import Logger
 import SBstoat.timeseriesPlotter as tp
 from SBstoat import namedTimeseries
 from SBstoat import rpickle
@@ -53,13 +53,16 @@ class ModelFitterCore(rpickle.RPickler):
 
     def __init__(self, modelSpecification, observedData,
                  parametersToFit=None,
-                 selectedColumns=None, method=METHOD_BOTH,
+                 selectedColumns=None,
+                 fitterMethods=[METHOD_BOTH],
+                 bootstrapMethods=[METHOD_LEASTSQ, METHOD_BOTH],
                  parameterLowerBound=PARAMETER_LOWER_BOUND,
                  parameterUpperBound=PARAMETER_UPPER_BOUND,
                  parameterDct={},
                  fittedDataTransformDct={},
                  logger=Logger(),
                  isPlot=True,
+                 _loggerPrefix="",
                  **bootstrapKwargs
                  ):
         """
@@ -88,8 +91,10 @@ class ModelFitterCore(rpickle.RPickler):
                    input: NamedTimeseries
                    output: array for the values of the column
         logger: Logger
-        method: str
-            method used for minimization
+        fitterMethods: str/list-str
+            method used for minimization in fitModel
+        bootstrapMethods: str/list-str
+            method used for minimization in bootstrap
         bootstrapKwargs: dict
             Parameters used in bootstrap
 
@@ -99,6 +104,7 @@ class ModelFitterCore(rpickle.RPickler):
         """
         if modelSpecification is not None:
             # Not the default constructor
+            self._loggerPrefix = _loggerPrefix
             self.modelSpecification = modelSpecification
             self.parametersToFit = parametersToFit
             self.lowerBound = parameterLowerBound
@@ -110,22 +116,25 @@ class ModelFitterCore(rpickle.RPickler):
             self.observedTS = observedData
             if self.observedTS is not None:
                 self.observedTS = mkNamedTimeseries(observedData)
-            # Identify the nan values in observed
-            self._observedTSIndexDct = {}
-            for col in self.observedTS.colnames:
-                self._observedTSIndexDct[col] = np.array(
-                      [i for i in range(len(self.observedTS))
-                      if np.isnan(self.observedTS[col][i])])
             #
             self.fittedDataTransformDct = fittedDataTransformDct
+            #
             if (selectedColumns is None) and (self.observedTS is not None):
                 selectedColumns = self.observedTS.colnames
             self.selectedColumns = selectedColumns
-            self._method = method
+            # Construct array of non-nan observed values
+            self._observedArr = self.observedTS[self.selectedColumns].flatten()
+            # Other internal state
+            self._fitterMethods = fitterMethods
+            if isinstance(self._fitterMethods, str):
+                self._fitterMethods = [self._fitterMethods]
+            self._bootstrapMethods = bootstrapMethods
+            if isinstance(self._bootstrapMethods, str):
+                self._bootstrapMethods = [self._bootstrapMethods]
             self._isPlot = isPlot
             self._plotter = tp.TimeseriesPlotter(isPlot=self._isPlot)
             self._plotFittedTS = None  # Timeseries that is plotted
-            self._logger = logger
+            self.logger = logger
             # The following are calculated during fitting
             self.roadrunnerModel = None
             self.minimizer = None  # lmfit.minimizer
@@ -155,8 +164,8 @@ class ModelFitterCore(rpickle.RPickler):
         """
         Overrides rpickler.
         """
-        if not "_logger" in self.__dict__.keys():
-            self._logger = Logger()
+        if not "logger" in self.__dict__.keys():
+            self.logger = Logger()
 
     def _validateFittedDataTransformDct(self):
         if self.fittedDataTransformDct is not None:
@@ -240,7 +249,7 @@ class ModelFitterCore(rpickle.RPickler):
         newSelectedColumns = list(self.selectedColumns)
         if len(missingNames) > 0:
             newObservedTS = observedTS.copy()
-            self._logger.exception("Missing names in antimony export: %s"
+            self.logger.exception("Missing names in antimony export: %s"
                   % str(missingNames))
             for name in observedTS.colnames:
                 missingName = "%s_" % name
@@ -252,7 +261,7 @@ class ModelFitterCore(rpickle.RPickler):
             newObservedTS = observedTS
         return newObservedTS, newSelectedColumns
 
-    def copy(self):
+    def copy(self, isKeepLogger=False):
         """
         Creates a copy of the model fitter.
         Preserves the user-specified settings and the results
@@ -262,7 +271,7 @@ class ModelFitterCore(rpickle.RPickler):
             try:
                 modelSpecification = self.modelSpecification.getAntimony()
             except Exception as err:
-                self._logger.error("Problem wth conversion to Antimony. Details:",
+                self.logger.error("Problem wth conversion to Antimony. Details:",
                       err)
                 raise ValueError("Cannot proceed.")
             observedTS, selectedColumns = self._adjustNames(
@@ -272,17 +281,24 @@ class ModelFitterCore(rpickle.RPickler):
             observedTS = self.observedTS.copy()
             selectedColumns = self.selectedColumns
         #
+        if isKeepLogger:
+            logger = self.logger
+        elif self.logger is not None:
+            logger = self.logger.copy()
+        else:
+            logger = None
         newModelFitter = self.__class__(
               copy.deepcopy(modelSpecification),
               observedTS,
               copy.deepcopy(self.parametersToFit),
               selectedColumns=selectedColumns,
-              method=self._method,
+              fitterMethods=self._fitterMethods,
+              bootstrapMethods=self._bootstrapMethods,
               parameterLowerBound=self.lowerBound,
               parameterUpperBound=self.upperBound,
               parameterDct=copy.deepcopy(self.parameterDct),
               fittedDataTransformDct=copy.deepcopy(self.fittedDataTransformDct),
-              logger=copy.deepcopy(self._logger),
+              logger=logger,
               isPlot=self._isPlot)
         if self.bootstrapResult is not None:
             newModelFitter.bootstrapResult = self.bootstrapResult.copy()
@@ -345,49 +361,87 @@ class ModelFitterCore(rpickle.RPickler):
                 return default
             else:
                 return parameter
-        #
+        ##V
+        block = Logger.join(self._loggerPrefix, "fitModel.simulate")
+        guid = self.logger.startBlock(block)
+        ## V
+        sub1Block = Logger.join(block, "sub1")
+        sub1Guid = self.logger.startBlock(sub1Block)
         startTime = set(self.observedTS.start, startTime)
         endTime = set(self.observedTS.end, endTime)
         numPoint = set(len(self.observedTS), numPoint)
+        ##  V
+        sub1aBlock = Logger.join(sub1Block, "sub1a")
+        sub1aGuid = self.logger.startBlock(sub1aBlock)
         if self.roadrunnerModel is None:
             self._initializeRoadrunnerModel()
         self.roadrunnerModel.reset()
+        ##  ^
+        self.logger.endBlock(sub1aGuid)
+        ##  V
+        sub1bBlock = Logger.join(sub1Block, "sub1b")
+        sub1bGuid = self.logger.startBlock(sub1bBlock)
         if params is not None:
           # Parameters have been specified
           self._setupModel(params)
+        ##  ^
+        self.logger.endBlock(sub1bGuid)
         # Do the simulation
         selectedColumns = list(self.selectedColumns)
         if not TIME in selectedColumns:
             selectedColumns.insert(0, TIME)
+        ## ^
+        self.logger.endBlock(sub1Guid)
+        ## V
+        roadrunnerBlock = Logger.join(block, "roadrunner")
+        roadrunnerGuid = self.logger.startBlock(roadrunnerBlock)
         data = self.roadrunnerModel.simulate(startTime, endTime, numPoint,
               selectedColumns)
+        self.logger.endBlock(roadrunnerGuid)
+        ## ^
         # Select the required columns
-        dataColnames = [s[1:-1] if s[0]=="[" else s for s in list(data.colnames)]
-        columnIndices = [i for i in range(len(data.colnames))
-              if data.colnames[i] in dataColnames]
-        data = data[:, columnIndices]
-        fittedTS = self._transformFittedTS(data)
+        ## V
+        sub2Block = Logger.join(block, "sub2")
+        sub2Guid = self.logger.startBlock(sub2Block)
+        fittedTS = NamedTimeseries(namedArray=data)
+        self.logger.endBlock(sub2Guid)
+        ## ^
+        self.logger.endBlock(guid)
+        ##^
         return fittedTS
         
-
-    def _simulate(self, **kwargs):
+    def updateFittedAndResiduals(self, **kwargs)->np.ndarray:
         """
-        Runs a simulation.
+        Updates values of self.fittedTS and self.residualsTS
+        based on self.params.
 
         Parameters
         ----------
         kwargs: dict
+            arguments for simulation
 
         Instance Variables Updated
         --------------------------
         self.fittedTS
+        self.residualsTS
+
+        Returns
+        -------
+        1-d ndarray of residuals
         """
-        self.fittedTS = self.simulate(**kwargs)
+        self.fittedTS = self.simulate(**kwargs)  # Updates self.fittedTS
+        cols = self.selectedColumns
+        if self.residualsTS is None:
+            self.residualsTS = self.observedTS.subsetColumns(cols)
+        self.residualsTS[cols] = self.observedTS[cols] - self.fittedTS[cols]
+        for col in cols:
+            self.residualsTS[col] = np.nan_to_num(self.residualsTS[col])
 
     def _residuals(self, params)->np.ndarray:
         """
         Compute the residuals between objective and experimental data
-        Handle nan values in observedTS.
+        Handle nan values in observedTS. This internal-only method
+        is implemented to maximize efficieency.
 
         Parameters
         ----------
@@ -402,16 +456,36 @@ class ModelFitterCore(rpickle.RPickler):
         -------
         1-d ndarray of residuals
         """
-        self._simulate(params=params)
-        cols = self.selectedColumns
-        if self.residualsTS is None:
-            self.residualsTS = self.observedTS.subsetColumns(cols)
-        self.residualsTS[cols] = self.observedTS[cols] - self.fittedTS[cols]
-        for col in cols:
-            if len(self._observedTSIndexDct[col]) > 0:
-                self.residualsTS[col][self._observedTSIndexDct[col]] = 0
-        #self.residualsTS[cols] = np.nan_to_num(self.residualsTS[cols], nan=0.0)
-        return self.residualsTS[cols].flatten()
+        block = Logger.join(self._loggerPrefix, "fitModel._residuals")
+        guid = self.logger.startBlock(block)
+        ##V
+        self.roadrunnerModel.reset()
+        self._setupModel(params)
+        #
+        roadrunnerBlock = Logger.join(block, "roadrunner")
+        roadrunnerGuid = self.logger.startBlock(roadrunnerBlock)
+        ## V
+        #
+        data = self.roadrunnerModel.simulate(
+              self.observedTS.start, self.observedTS.end,
+              len(self.observedTS),
+              self.selectedColumns)
+        ## ^
+        self.logger.endBlock(roadrunnerGuid)
+        #
+        tailBlock = Logger.join(block, "tail")
+        tailGuid = self.logger.startBlock(tailBlock)
+        ## V
+        residualsArr = self._observedArr - data.flatten()
+        residualsArr = np.nan_to_num(residualsArr)
+        ## ^
+        self.logger.endBlock(tailGuid)
+        ##^
+        self.logger.endBlock(guid)
+        #
+        if np.isnan(np.sum(residualsArr)):
+            import pdb; pdb.set_trace()
+        return residualsArr
 
     def fitModel(self, params:lmfit.Parameters=None,
           max_nfev:int=100):
@@ -429,6 +503,8 @@ class ModelFitterCore(rpickle.RPickler):
         -------
         f.fitModel()
         """
+        block = Logger.join(self._loggerPrefix, "fitModel")
+        guid = self.logger.startBlock(block)
         self._initializeRoadrunnerModel()
         if self.parametersToFit is None:
             # Compute fit and residuals for base model
@@ -445,26 +521,34 @@ class ModelFitterCore(rpickle.RPickler):
             #   Global differential evolution to get us close to minimum
             #   A local Levenberg-Marquardt to getsus to the minimum
             isMinimized = False
-            if self._method in [METHOD_BOTH, METHOD_DIFFERENTIAL_EVOLUTION]:
-                minimizer = lmfit.Minimizer(self._residuals, params,
-                      max_nfev=max_nfev)
-                self.minimizerResult = minimizer.minimize(
-                      method=METHOD_DIFFERENTIAL_EVOLUTION,
-                      max_nfev=max_nfev)
-                params_DE = self.minimizerResult.params
-                residuals_DE = self._residuals(params=params_DE)
-                isMinimized = True
-            if self._method in [METHOD_BOTH, METHOD_LEASTSQ]:
-                minimizer = lmfit.Minimizer(self._residuals, params,
-                      max_nfev=max_nfev)
-                self.minimizerResult = minimizer.minimize(
-                      method=METHOD_LEASTSQ,
-                      max_nfev=max_nfev)
-                params_LS = self.minimizerResult.params
-                residuals_LS = self._residuals(params=params_LS)
-                isMinimized = True
+            for method in self._fitterMethods:           
+                if method in [METHOD_BOTH, METHOD_DIFFERENTIAL_EVOLUTION]:
+                    minimizer = lmfit.Minimizer(self._residuals, params,
+                          max_nfev=max_nfev)
+                    try:
+                        self.minimizerResult = minimizer.minimize(
+                              method=METHOD_DIFFERENTIAL_EVOLUTION,
+                              max_nfev=max_nfev)
+                        params_DE = self.minimizerResult.params
+                        residuals_DE = self._residuals(params=params_DE)
+                        isMinimized = True
+                    except Exception as excp:
+                        self.logger.error(
+                              "Minimizer failed using differential evolution", excp)
+                        method = METHOD_LEASTSQ
+                if method in [METHOD_BOTH, METHOD_LEASTSQ]:
+                    minimizer = lmfit.Minimizer(self._residuals, params,
+                          max_nfev=max_nfev)
+                    self.minimizerResult = minimizer.minimize(
+                          method=METHOD_LEASTSQ,
+                          max_nfev=max_nfev)
+                    params_LS = self.minimizerResult.params
+                    residuals_LS = self._residuals(params=params_LS)
+                    isMinimized = True
+                if isMinimized:
+                    break
             if not isMinimized:
-                raise ValueError("Invalid method specified: %s" % self._method)
+                raise ValueError("Minimizer failed")
             if np.std(residuals_DE) <= np.std(residuals_LS):
                 self.params = params_DE
             else:
@@ -477,7 +561,8 @@ class ModelFitterCore(rpickle.RPickler):
                 msg = "*** Minimizer failed for this model and data."
                 raise ValueError(msg)
         # Ensure that residualsTS and fittedTS match the parameters
-        _ = self._residuals(params=self.params)
+        self.updateFittedAndResiduals(params=self.params)
+        self.logger.endBlock(guid)
 
     def getFittedModel(self):
         """
@@ -508,7 +593,7 @@ class ModelFitterCore(rpickle.RPickler):
             except Exception as err:
                 msg = "_modelFitterCore/_setupModel: Could not set value for %s"  \
                       % parameter
-                self._logger.error(msg, err)
+                self.logger.error(msg, err)
 
     def mkParams(self, parameterDct:dict=None)->lmfit.Parameters:
         """
@@ -551,7 +636,7 @@ class ModelFitterCore(rpickle.RPickler):
                 except Exception as err:
                     msg = "modelFitterCore/mkParams parameterName %s" \
                           % parameterName
-                    self._logger.error(msg, err)
+                    self.logger.error(msg, err)
             else:
                 value = np.mean([self.lowerBound, self.upperBound])
                 params.add(parameterName, value=value,
