@@ -32,6 +32,8 @@ Usage:
    timeseries["S9"] = 10  # Assign a constant to all rows
 """
 
+from SBstoat import rpickle
+
 import collections
 import copy
 import csv
@@ -100,14 +102,15 @@ def cleanColnames(colnames):
 
 
 ################## CLASSES ########################
-class NamedTimeseries(object):
+class NamedTimeseries(rpickle.RPickler):
           
     def __init__(self,
           csvPath=None,
           colnames=None, array=None,
           namedArray=None,
           dataframe=None,
-          timeseries=None):
+          timeseries=None,
+          isNone=False):
         """
         Parameters
         ---------
@@ -120,6 +123,8 @@ class NamedTimeseries(object):
         dataframe: pd.DataFrame
             index: time
         timeseries: NamedTimeseries
+        isNone: bool
+            Initialization to None values
                
        Usage
        -----
@@ -130,6 +135,8 @@ class NamedTimeseries(object):
            At one of the following most be non-None:
              csvPath, colnames & array, dataframe, timeseries
         """
+        if isNone:
+            return
         if timeseries is not None:
             # Copy the existing object
             for k in timeseries.__dict__.keys():
@@ -157,6 +164,8 @@ class NamedTimeseries(object):
             timeIdxs = [i for i, c in enumerate(allColnames) if c.lower() == TIME]
             if len(timeIdxs) != 1:
                 raise ValueError("Must have exactly one time column")
+            self.values = np.array(self.values)  # Ensure not NamedArray
+            self.values = self.values.astype(float)
             allColnames[timeIdxs[0]] = TIME
             self.allColnames = []  # all column names
             self.colnames = []  # Names of non-time columns
@@ -165,8 +174,20 @@ class NamedTimeseries(object):
             times = self.values[:, self._indexDct[TIME]]
             self.start = min(times)
             self.end = max(times)
+    
+    @classmethod
+    def rpConstruct(cls):
+        """
+        Overrides rpickler.rpConstruct to create a method that
+        constructs an instance without arguments.
+        
+        Returns
+        -------
+        Instance of cls
+        """
+        return cls(isNone=True)
 
-    def __str__(self):
+    def __repr__(self):
         df = self.to_dataframe()
         return str(df)
 
@@ -174,7 +195,8 @@ class NamedTimeseries(object):
         self.allColnames.append(name)
         self._indexDct[name] = self.allColnames.index(name)
         self.colnames = list(self.allColnames)
-        self.colnames.remove(TIME)
+        if TIME in self.colnames:
+            self.colnames.remove(TIME)
 
     def __setitem__(self, reference, value):
         """
@@ -238,8 +260,7 @@ class NamedTimeseries(object):
             if isinstance(reference, str):
                 indices = self._indexDct[reference]
             elif isinstance(reference, list):
-                indices = [v for k, v in self._indexDct.items()
-                      if k in reference]
+                indices = [v for k, v in self._indexDct.items() if k in reference]
             else:
                 raise (KeyKerror)
         except KeyError:
@@ -279,15 +300,18 @@ class NamedTimeseries(object):
         def handleStr(reference):
             indices = self._getColumnIndices(reference)
             return self.values[:, indices]
-        # indicate types
+        # Handle common case first
+        if isinstance(reference, str):
+            if reference.lower() == TIME:
+                reference = TIME
+            indices = self._indexDct[reference]
+            return self.values[:, indices]
+        # Do more complex types
         isInt = False
         isListInt = False
         isSlice = False
-        isStr = False
         isListStr = False
-        if isinstance(reference, str):
-            isStr = True
-        elif isinstance(reference, list):
+        if isinstance(reference, list):
             if isinstance(reference[0], str):
                 isListStr = True
             elif isinstance(reference[0], int):
@@ -297,10 +321,6 @@ class NamedTimeseries(object):
         elif isinstance(reference, slice):
             isSlice = True
         # Process row type references
-        if isStr:
-            if reference.lower() == TIME:
-                reference = TIME
-            return handleStr(reference)
         if isListStr:
             timeStrs = [c for c in reference if c.lower() == TIME]
             if len(timeStrs) == 1:
@@ -331,25 +351,6 @@ class NamedTimeseries(object):
     def __len__(self):
        return np.shape(self.values)[0]
        
-    def _load(self, filePath):
-        """
-        Load data from a CSV file.
-        
-        Parameters
-        ----------
-        
-        filePath: str
-              Path to the file containing time series data. Data should be arranged
-              in columns, first colums representing time, remaining columns will
-              be whatever timeseries data is avaialble.
-        selectedVariables : list of strings
-        """
-        indices = self._getColumnIndices(reference)
-        return self.values[:, indices]
-
-    def __len__(self):
-       return np.shape(self.values)[0]
-       
     def _load(self, filePath):
         """
         Load data from a CSV file.
@@ -422,10 +423,37 @@ class NamedTimeseries(object):
         array = self.values[rowIdxs, :]
         return array[:, colIdxs]
 
-    def copy(self):
-        return copy.deepcopy(self)
+    def copy(self, isInitialize=False):
+        """
+        Copies the orginal timeseries with an option for initializing columns
+        to 0.
+
+        Parameters
+        ----------
+        isIinitialize: bool
+
+        Return
+        -----
+        NamedTimeseries
+        """
+        ts = copy.deepcopy(self)
+        if isInitialize:
+            for column in self.colnames:
+                ts[column] = 0.0
+        return ts
 
     def equals(self, other):
+        """
+        Checks if two timeseries have the same columns and values.
+
+        Parameters
+        ----------
+        other: NamedTimeseries
+        
+        Returns
+        -------
+        bool
+        """
         diff = set(self.allColnames).symmetric_difference(other.allColnames)
         if len(diff) > 0:
             return False
@@ -612,3 +640,22 @@ class NamedTimeseries(object):
                 raise RuntimeError("Should not get here")
             last = value
         return times
+
+    def rename(self, curName, newName):
+        """
+        Creates a new timeseries, renaming the specified column.
+
+        Parameters
+        ----------
+        curName: str
+           Name of the column to change
+        newName: str
+           New name of the column to change
+        """
+        if not curName in self._indexDct.keys():
+            msg = "Invalid reference to NamedTimeseries. No column %s" % curName
+            raise ValueError(msg)
+        newTimeseries = self.copy()
+        newTimeseries[newName] = self[curName]
+        del newTimeseries[curName]
+        return newTimeseries
