@@ -147,14 +147,24 @@ class TimeseriesPlotter(object):
 
     @Expander(po.KWARGS, po.BASE_OPTIONS, excludes=[po.TITLE], indent=8,
            header=po.HEADER)
-    def plotTimeSingle(self, timeseries1, **kwargs):
+    def plotTimeSingle(self, timeseries1:NamedTimeseries,
+           meanTS:NamedTimeseries=None, stdTS:NamedTimeseries=None,
+           bandLowTS:NamedTimeseries=None, bandHighTS:NamedTimeseries=None,
+           ax_spec=None,
+           position=None,  **kwargs):
         """
         Constructs plots of single columns, possibly with a second
         timeseries.
 
         Parameters
         ---------
-        timeseries1: NamedTimeseries
+        timeseries1: timeseries to plot
+        meanTS: mean values of timeseries to plot
+        stdTS: standard deviations of meanTS (same times)
+        bandLowTS: timeseries that describes the lower band for timeseries1
+        bandhighTS: timeseries that describes the higher band for timeseries1
+        ax_spec: Optionally specified axis for all lines
+        position: (int, int) - position of ax_spec
         #@expand
                
         Example
@@ -163,24 +173,6 @@ class TimeseriesPlotter(object):
         plotter.plotTimeSingle(timeseries)
         """
         options = self._mkPlotOptionsMatrix(timeseries1, **kwargs)
-        def mkStatement(ax, timeseries:NamedTimeseries, variable:str, lineIdx:int):
-            markers = options.get(po.MARKER, lineIdx=lineIdx)
-            if markers is not None:
-                if isinstance(markers, list):
-                    marker = markers[lineIdx]
-                else:
-                    marker = markers
-            else:
-                marker = None
-            if marker is None:
-                statement = StatementManager(ax.plot)
-            else:
-                statement = StatementManager(ax.scatter)
-            #
-            statement.addPosarg(timeseries[TIME])
-            statement.addPosarg(timeseries[variable])
-            return statement
-        #
         # Adjust rows and columns
         numPlot = len(options.columns)  # Number of plots
         if po.NUM_COL in kwargs.keys():
@@ -189,29 +181,120 @@ class TimeseriesPlotter(object):
             options.numCol = int(np.ceil(numPlot/options.numRow))
         options.set(po.XLABEL, TIME)
         # Create the LayoutManager
-        layout = self._mkManager(options, numPlot)
+        if ax_spec is None:
+            layout = self._mkManager(options, numPlot)
+        else:
+            layout = None
+        #
+        def mkStatement(ax, statement:StatementManager,
+            timeseries:NamedTimeseries, variable:str):
+            """
+            Creates a plotting statement.
+   
+            Parameters
+            ----------
+            ax: plotting axes
+            initialStatement: statement if initialized
+            timeseries: what to plot
+            variable: variable to plot
+            
+            Returns
+            -------
+            StatementManager
+            """
+            times, values = self._adjustSeriesForNan(
+                  timeseries[TIME], timeseries[variable])
+            statement.addPosarg(times)
+            statement.addPosarg(values)
+            return statement
+        #
+        def isFirstColumn(plotIdx):
+            if layout is not None:
+                return layout.isFirstColumn(plotIdx)
+            else:
+                if position is not None:
+                    return position[1]  == 0
+                else:
+                    return True
+        #
+        def isLastRow(plotIdx):
+            if layout is not None:
+                return layout.isLastRow(plotIdx)
+            else:
+                if po.NUM_ROW in kwargs.keys():
+                    numRow = kwargs[po.NUM_ROW]
+                else:
+                    return True
+                if position is None:
+                    return True
+                return position[0] + 1 == numRow
+        #
+        def mkBand(ts, variable, plotIdx, lineIdx):
+            initialStatement = StatementManager(ax.fill_between)
+            statement = mkStatement(ax, initialStatement, timeseries1, variable)
+            statement.addPosarg(ts[variable])
+            statement.addKwargs(alpha=0.2)
+            options.do(ax, statement=statement, plotIdx=plotIdx, lineIdx=lineIdx)
+        #
         # Construct the plots
         baseOptions = copy.deepcopy(options)
         for plotIdx, variable in enumerate(options.columns):
-            ax = layout.getAxis(plotIdx)
+            if ax_spec is None:
+                ax = layout.getAxis(plotIdx)
+            else:
+                ax = ax_spec
             options = copy.deepcopy(baseOptions)
             #ax = axes[row, col]
             options.set(po.YLABEL, "concentration")
-            options.title = variable
-            if not layout.isFirstColumn(plotIdx):
+            options.set(po.TITLE, variable)
+            if not isFirstColumn(plotIdx):
                 options.ylabel =  NULL_STR
                 options.set(po.YLABEL, "", isOverride=True)
-            if not layout.isLastRow(plotIdx):
+            if not isLastRow(plotIdx):
                 options.set(po.XLABEL, "", isOverride=True)
             # Construct the plot
             lineIdx = 0
-            statement = mkStatement(ax, timeseries1, variable, lineIdx)
+            initialStatement = StatementManager(ax.plot)
+            statement = mkStatement(ax, initialStatement, timeseries1, variable)
             options.do(ax, statement=statement, plotIdx=plotIdx, lineIdx=lineIdx)
+            legends = []
             if options.timeseries2 is not None:
                 lineIdx = 1
-                statement = mkStatement(ax, options.timeseries2, variable, lineIdx)
-                options.set(po.LEGEND, [LABEL1, LABEL2])
+                initialStatement = StatementManager(ax.plot)
+                if options.marker is not None:
+                    if len(options.marker) > 1:
+                        if options.marker[lineIdx] is not None:
+                            initialStatement = StatementManager(ax.scatter)
+                    elif options.marker is not None:
+                        initialStatement = StatementManager(ax.scatter)
+                statement = mkStatement(ax, initialStatement,
+                      options.timeseries2, variable)
+                legends = [LABEL1, LABEL2]
+                options.lengends = legends
                 options.do(ax, statement=statement, plotIdx=plotIdx, lineIdx=lineIdx)
+            if  meanTS is not None:
+                if stdTS is None:
+                    stdTS = meanTS.copy(isInitialize=True)
+                lineIdx = 2
+                initialStatement = StatementManager(ax.scatter)
+                statement = mkStatement(ax, initialStatement, meanTS, variable)
+                options.marker = "^"
+                statement.addKwargs(facecolors="none")
+                options.do(ax, statement=statement, plotIdx=plotIdx, lineIdx=lineIdx)
+                #
+                initialStatement = StatementManager(ax.errorbar)
+                statement = mkStatement(ax, initialStatement, meanTS, variable)
+                statement.addKwargs(yerr=stdTS[variable])
+                statement.addKwargs(linestyle="none")
+                legends.append("Bootstrap mean")
+                options.lengends = legends
+                options.do(ax, statement=statement, plotIdx=plotIdx, lineIdx=lineIdx)
+            if  bandLowTS is not None:
+                lineIdx = 0
+                mkBand(bandLowTS, variable, plotIdx, lineIdx)
+            if  bandHighTS is not None:
+                lineIdx = 0
+                mkBand(bandHighTS, variable, plotIdx, lineIdx)
         if self.isPlot:
             plt.show()
 
@@ -249,8 +332,10 @@ class TimeseriesPlotter(object):
                     statement = StatementManager(ax.plot)
                 else:
                     statement = StatementManager(ax.scatter)
-                statement.addPosarg(timeseries[TIME])
-                statement.addPosarg(timeseries[col])
+                times, values = self._adjustSeriesForNan(
+                      timeseries[TIME], timeseries[col])
+                statement.addPosarg(times)
+                statement.addPosarg(values)
                 options.legend = timeseries.colnames
                 options.do(ax, statement=statement, plotIdx=plotIdx, lineIdx=lineIdx)
         #
@@ -547,3 +632,7 @@ class TimeseriesPlotter(object):
                   lineIdx, plotIdx=plotIdx)
         if self.isPlot:
             plt.show()
+
+    def _adjustSeriesForNan(self, times, values):
+        indices = [i for i in range(len(values)) if not np.isnan(values[i])]
+        return times[indices], values[indices]
